@@ -43,7 +43,17 @@ static PLpgSQL_plugin plugin_funcs = { plpgsql_check_profiler_func_init,
 									   plpgsql_check_profiler_stmt_beg,
 									   plpgsql_check_profiler_stmt_end,
 									   NULL,
-									   NULL};
+									   NULL
+
+#if PG_VERSION_NUM >= 150000
+
+									  , NULL,
+									  NULL,
+									  NULL
+
+#endif
+
+									  };
 
 
 static const struct config_enum_entry plpgsql_check_mode_options[] = {
@@ -75,8 +85,19 @@ static const struct config_enum_entry tracer_level_options[] = {
 };
 
 void			_PG_init(void);
+
+#if PG_VERSION_NUM < 150000
+
 void			_PG_fini(void);
 
+#endif
+
+#if PG_VERSION_NUM >= 150000
+
+shmem_request_hook_type prev_shmem_request_hook = NULL;
+
+#endif
+
 shmem_startup_hook_type prev_shmem_startup_hook = NULL;
 
 bool plpgsql_check_regress_test_mode;
@@ -290,14 +311,30 @@ _PG_init(void)
 						    PGC_POSTMASTER, 0,
 						    NULL, NULL, NULL);
 
+#if PG_VERSION_NUM < 150000
+
+		/*
+		 * If you change code here, don't forget to also report the
+		 * modifications in plpgsql_check_profiler_shmem_request() for pg15 and
+		 * later.
+		 */
 		RequestAddinShmemSpace(plpgsql_check_shmem_size());
 
 		RequestNamedLWLockTranche("plpgsql_check profiler", 1);
 		RequestNamedLWLockTranche("plpgsql_check fstats", 1);
 
+#endif
+
 		/*
 		 * Install hooks.
 		 */
+#if PG_VERSION_NUM >= 150000
+
+		prev_shmem_startup_hook = shmem_startup_hook;
+		shmem_request_hook = plpgsql_check_profiler_shmem_request;
+
+#endif
+
 		prev_shmem_startup_hook = shmem_startup_hook;
 		shmem_startup_hook = plpgsql_check_profiler_shmem_startup;
 	}
@@ -311,12 +348,19 @@ _PG_init(void)
 	inited = true;
 }
 
+#if PG_VERSION_NUM < 150000
+
 /*
  * Module unload callback
  */
 void
 _PG_fini(void)
 {
+#if PG_VERSION_NUM >= 150000
+
+	shmem_request_hook = prev_shmem_request_hook;
+
+#endif
 	shmem_startup_hook = prev_shmem_startup_hook;
 
 	/* Be more correct, and clean rendezvous variable */
@@ -325,3 +369,5 @@ _PG_fini(void)
 	needs_fmgr_hook = plpgsql_check_next_needs_fmgr_hook;
 	fmgr_hook = plpgsql_check_next_fmgr_hook;
 }
+
+#endif
