@@ -15,6 +15,10 @@ typedef uint32 pc_queryid;
 #define NOQUERYID	(0)
 #endif
 
+#if PG_VERSION_NUM < 150000
+#define parse_analyze_fixedparams	parse_analyze
+#endif
+
 enum
 {
 	PLPGSQL_CHECK_ERROR,
@@ -67,6 +71,7 @@ typedef struct PLpgSQL_stmt_stack_item
 	PLpgSQL_stmt	   *stmt;
 	char			   *label;
 	struct PLpgSQL_stmt_stack_item *outer;
+	bool				is_exception_handler;
 } PLpgSQL_stmt_stack_item;
 
 typedef struct plpgsql_check_result_info
@@ -176,6 +181,8 @@ extern void plpgsql_check_assign_tupdesc_row_or_rec(PLpgSQL_checkstate *cstate,
 extern void plpgsql_check_recval_assign_tupdesc(PLpgSQL_checkstate *cstate, PLpgSQL_rec *rec, TupleDesc tupdesc, bool is_null);
 extern void plpgsql_check_recval_init(PLpgSQL_rec *rec);
 extern void plpgsql_check_recval_release(PLpgSQL_rec *rec);
+extern void plpgsql_check_is_assignable(PLpgSQL_execstate *estate, int dno);
+
 
 /*
  * functions from format.c
@@ -203,6 +210,7 @@ extern void plpgsql_check_precheck_conditions(plpgsql_check_info *cinfo);
 extern char *plpgsql_check_get_src(HeapTuple procTuple);
 extern Oid plpgsql_check_pragma_func_oid(void);
 extern bool plpgsql_check_is_plpgsql_function(Oid foid);
+extern Oid plpgsql_check_get_op_namespace(Oid opno);
 
 /*
  * functions from tablefunc.c
@@ -313,6 +321,9 @@ extern int plpgsql_check_profiler_max_shared_chunks;
 extern needs_fmgr_hook_type		plpgsql_check_next_needs_fmgr_hook;
 extern fmgr_hook_type			plpgsql_check_next_fmgr_hook;
 
+#if PG_VERSION_NUM >= 150000
+extern void plpgsql_check_profiler_shmem_request(void);
+#endif
 extern void plpgsql_check_profiler_shmem_startup(void);
 
 extern Size plpgsql_check_shmem_size(void);
@@ -378,6 +389,9 @@ extern bool plpgsql_check_runtime_pragma_vector_changed;
  * functions from plpgsql_check.c
  */
 
+#if PG_VERSION_NUM >= 150000
+extern shmem_request_hook_type prev_shmem_request_hook;
+#endif
 extern shmem_startup_hook_type prev_shmem_startup_hook;
 
 extern PLpgSQL_plugin **plpgsql_check_plugin_var_ptr;
