@@ -606,6 +606,28 @@ plpgsql_check_shmem_size(void)
 	return num_bytes;
 }
 
+/*
+ * Request additional shared memory resources.
+ *
+ * If you change code here, don't forget to also report the modifications in
+ * _PG_init() for pg14 and below.
+ */
+#if PG_VERSION_NUM >= 150000
+
+void
+plpgsql_check_profiler_shmem_request(void)
+{
+	if (prev_shmem_request_hook)
+		prev_shmem_request_hook();
+
+	RequestAddinShmemSpace(plpgsql_check_shmem_size());
+
+	RequestNamedLWLockTranche("plpgsql_check profiler", 1);
+	RequestNamedLWLockTranche("plpgsql_check fstats", 1);
+}
+
+#endif
+
 /*
  * Initialize shared memory used like permanent profile storage.
  * No other parts use shared memory, so this code is completly here.
@@ -2160,7 +2182,7 @@ profiler_get_dyn_queryid(PLpgSQL_execstate *estate, PLpgSQL_expr *expr, query_pa
 		snapshot_set = true;
 	}
 
-	query = parse_analyze(parsetree, query_string, paramtypes, nparams, NULL);
+	query = parse_analyze_fixedparams(parsetree, query_string, paramtypes, nparams, NULL);
 
 	if (snapshot_set)
 		PopActiveSnapshot();
