@@ -24,6 +24,7 @@ static void check_stmts(PLpgSQL_checkstate *cstate, List *stmts, int *closing, L
 static PLpgSQL_stmt_stack_item * push_stmt_to_stmt_stack(PLpgSQL_checkstate *cstate);
 static void pop_stmt_from_stmt_stack(PLpgSQL_checkstate *cstate);
 static bool is_any_loop_stmt(PLpgSQL_stmt *stmt);
+static bool is_inside_exception_handler(PLpgSQL_stmt_stack_item *current);
 static PLpgSQL_stmt * find_nearest_loop(PLpgSQL_stmt_stack_item *current);
 static PLpgSQL_stmt * find_stmt_with_label(char *label, PLpgSQL_stmt_stack_item *current);
 static int possibly_closed(int c);
@@ -43,6 +44,12 @@ static void check_dynamic_sql(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, PL
 
 #if PG_VERSION_NUM >= 110000
 
+static bool is_inside_protected_block(PLpgSQL_stmt_stack_item *current);
+
+#endif
+
+#if PG_VERSION_NUM >= 110000
+
 static void
 check_variable(PLpgSQL_checkstate *cstate, PLpgSQL_variable *var)
 {
@@ -285,6 +292,8 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 						int		closing_handlers = PLPGSQL_CHECK_UNKNOWN;
 						List   *exceptions_transformed = NIL;
 
+						cstate->top_stmt_stack->is_exception_handler = true;
+
 						if (*closing == PLPGSQL_CHECK_CLOSED_BY_EXCEPTIONS)
 						{
 							ListCell   *lc;
@@ -820,6 +829,8 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 				{
 					PLpgSQL_stmt_return *stmt_rt = (PLpgSQL_stmt_return *) stmt;
 
+					*closing = PLPGSQL_CHECK_CLOSED;
+
 					if (stmt_rt->retvarno >= 0)
 					{
 						PLpgSQL_datum *retvar = cstate->estate->datums[stmt_rt->retvarno];
@@ -843,7 +854,10 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 								{
 									PLpgSQL_rec *rec = (PLpgSQL_rec *) retvar;
 
-									if (recvar_tupdesc(rec) && estate->rsi && IsA(estate->rsi, ReturnSetInfo))
+									/* don't do next check, when result tuple desc is fake */
+									if (recvar_tupdesc(rec) &&
+										!cstate->fake_rtd &&
+										estate->rsi && IsA(estate->rsi, ReturnSetInfo))
 									{
 										TupleDesc	rettupdesc = estate->rsi->expectedDesc;
 										TupleConversionMap *tupmap ;
@@ -861,7 +875,9 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 								{
 									PLpgSQL_row *row = (PLpgSQL_row *) retvar;
 
-									if (row->rowtupdesc && estate->rsi && IsA(estate->rsi, ReturnSetInfo))
+									if (row->rowtupdesc &&
+										!cstate->fake_rtd &&
+										estate->rsi && IsA(estate->rsi, ReturnSetInfo))
 									{
 										TupleDesc	rettupdesc = estate->rsi->expectedDesc;
 										TupleConversionMap *tupmap ;
@@ -880,8 +896,6 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 						}
 					}
 
-					*closing = PLPGSQL_CHECK_CLOSED;
-
 					if (stmt_rt->expr)
 						plpgsql_check_returned_expr(cstate, stmt_rt->expr, true);
 				}
@@ -1206,6 +1220,8 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 						plpgsql_check_expr(cstate, (PLpgSQL_expr *) lfirst(l));
 					}
 
+					plpgsql_check_target(cstate, stmt_open->curvar, NULL, NULL);
+
 					cstate->modif_variables = bms_add_member(cstate->modif_variables,
 									 stmt_open->curvar);
 				}
@@ -1222,6 +1238,14 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 
 						plpgsql_check_target(cstate, diag_item->target, NULL, NULL);
 					}
+
+					if (stmt_getdiag->is_stacked &&
+						!is_inside_exception_handler(outer_stmt))
+					{
+						ereport(ERROR,
+								(errcode(ERRCODE_STACKED_DIAGNOSTICS_ACCESSED_WITHOUT_ACTIVE_HANDLER),
+								  errmsg("GET STACKED DIAGNOSTICS cannot be used outside an exception handler")));
+					}
 				}
 				break;
 
@@ -1278,6 +1302,18 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 					ereport(ERROR,
 							(errcode(ERRCODE_INVALID_TRANSACTION_TERMINATION),
 							 errmsg("invalid transaction termination")));
+
+				if (is_inside_protected_block(outer_stmt))
+				{
+					if (stmt->cmd_type == PLPGSQL_STMT_COMMIT)
+						ereport(ERROR,
+								(errcode(ERRCODE_INVALID_TRANSACTION_TERMINATION),
+								 errmsg("cannot commit while a subtransaction is active")));
+					else
+						ereport(ERROR,
+								(errcode(ERRCODE_INVALID_TRANSACTION_TERMINATION),
+								 errmsg("cannot roll back while a subtransaction is active")));
+				}
 				break;
 
 			case PLPGSQL_STMT_CALL:
@@ -1297,6 +1333,7 @@ plpgsql_check_stmt(PLpgSQL_checkstate *cstate, PLpgSQL_stmt *stmt, int *closing,
 					if (target != NULL)
 					{
 						check_variable(cstate, (PLpgSQL_variable *) target);
+
 						plpgsql_check_assignment_to_variable(cstate, stmt_call->expr,
 																(PLpgSQL_variable *) target, -1);
 
@@ -1448,7 +1485,7 @@ push_stmt_to_stmt_stack(PLpgSQL_checkstate *cstate)
 	PLpgSQL_stmt_stack_item *stmt_stack_item;
 	PLpgSQL_stmt_stack_item *current = cstate->top_stmt_stack;
 
-	stmt_stack_item = (PLpgSQL_stmt_stack_item *) palloc(sizeof(PLpgSQL_stmt_stack_item));
+	stmt_stack_item = (PLpgSQL_stmt_stack_item *) palloc0(sizeof(PLpgSQL_stmt_stack_item));
 	stmt_stack_item->stmt = stmt;
 
 	switch (PLPGSQL_STMT_TYPES stmt->cmd_type)
@@ -1564,6 +1601,55 @@ find_nearest_loop(PLpgSQL_stmt_stack_item *current)
 	return NULL;
 }
 
+#if PG_VERSION_NUM >= 110000
+
+/*
+ * Returns true, when some outer block handles exceptions.
+ * It is used for check of correct usage of COMMIT or ROLLBACK.
+ */
+static bool
+is_inside_protected_block(PLpgSQL_stmt_stack_item *current)
+{
+	while (current != NULL)
+	{
+		if (current->stmt->cmd_type == PLPGSQL_STMT_BLOCK)
+		{
+			PLpgSQL_stmt_block *stmt_block = (PLpgSQL_stmt_block *) current->stmt;
+
+			if (stmt_block->exceptions && !current->is_exception_handler)
+				return true;
+		}
+
+		current = current->outer;
+	}
+
+	return false;
+}
+
+#endif
+
+/*
+ * This is used for check of correct usage GET STACKED DIAGNOSTICS
+ */
+static bool
+is_inside_exception_handler(PLpgSQL_stmt_stack_item *current)
+{
+	while (current != NULL)
+	{
+		if (current->stmt->cmd_type == PLPGSQL_STMT_BLOCK)
+		{
+			PLpgSQL_stmt_block *stmt_block = (PLpgSQL_stmt_block *) current->stmt;
+
+			if (stmt_block->exceptions && current->is_exception_handler)
+				return true;
+		}
+
+		current = current->outer;
+	}
+
+	return false;
+}
+
 /*
  * returns false, when a variable doesn't shadows any other variable
  */
@@ -1900,7 +1986,7 @@ check_dynamic_sql(PLpgSQL_checkstate *cstate,
 							 * so most safe value is NULL instead.
 							 */
 							appendStringInfo(&sinfo, " null ");
-							found_literal_placeholder = false;
+							found_literal_placeholder = true;
 							expr_is_const = false;
 						}
 						else
